@@ -133,16 +133,17 @@ def make_group_average(subjects=None, subject_dir=None, average=None, hemi="lh",
         # start all subjects registrations to .tif* in parallel
         for subject in subjects:
             # print "{}/surf/{}.sphere {}{}.tif {}.sphere.myreg{}".format(subject, hemi, template, i, hemi, i)
-            # print [command, "{}/surf/{}.sphere {}{}.tif {}.sphere.myreg{}".format(subject, hemi, template, i, hemi, i)]
+            # print [command, "{}/surf/{}.sphere {}{}.tif {}.sphere.myreg{}".format(subject,
+            # hemi, template, i, hemi, i)]
             processes.add(subprocess.Popen(
-                ["{} {}/surf/lh.sphere lh{}{}.tif {}.sphere.{}{}".format(command,
-                                                                       os.path.split(subject)[1],
-                                                                       template, i, "lh", template, i)],
+                [f'{command} '
+                 f'{os.path.split(subject)[1]}{os.sep}surf{os.sep}lh.sphere lh{template}{i}.tif '
+                 f'{"lh"}.sphere.{template}{i}'],
                 shell=True))
             processes.add(subprocess.Popen(
-                ["{} {}/surf/rh.sphere rh{}{}.tif {}.sphere.{}{}".format(command,
-                                                                       os.path.split(subject)[1],
-                                                                       template, i, "rh", template, i)],
+                [f"{command} "
+                 f"{os.path.split(subject)[1]}{os.sep}surf{os.sep}rh.sphere rh{template}{i}.tif "
+                 f"{'rh'}.sphere.{template}{i}"],
                 shell=True))
 
             # check if n_cpu processes are reached
@@ -152,7 +153,8 @@ def make_group_average(subjects=None, subject_dir=None, average=None, hemi="lh",
 
             #        for subject in subjects:
             #           print "        mris_register step {}{}".format(i, subject)
-            #         command = "FREESURFER mris_register {}/surf/{}.sphere {}{}.tif {}.sphere.myreg{}".format(subject,hemi,template,i,hemi,i)
+            #         command = "FREESURFER mris_register {}/surf/{}.sphere {}{}.tif
+            #         {}.sphere.myreg{}".format(subject,hemi,template,i,hemi,i)
             #       print command
             #     os.system(command)
         # needed? wait for all processes to finish
@@ -162,23 +164,15 @@ def make_group_average(subjects=None, subject_dir=None, average=None, hemi="lh",
 
         short_subjects = [os.path.split(subject)[1] for subject in subjects]
         print("        making template for step {}".format(i))
-      # command = "FREESURFER mris_make_template {} sphere {} rh{}0.tif".format("rh", os.path.split(average)[1],
+        # command = "FREESURFER mris_make_template {} sphere {} rh{}0.tif".format("rh", os.path.split(average)[1],
         #                                                                         template)
-        command = "FREESURFER mris_make_template {} sphere.{}{} {} rh{}{}.tif".format("rh",
-                                                                                      template,
-                                                                                      i,
-                                                                                      " ".join(short_subjects),
-                                                                                      template,
-                                                                                      i + 1)
+        command = f"FREESURFER mris_make_template {'rh'} sphere.{template}{i} {' '.join(short_subjects)} " \
+                  f"rh{template}{i + 1}.tif"
         print(command)
         os.system(command)
 
-        command = "FREESURFER mris_make_template {} sphere.{}{} {} lh{}{}.tif".format("lh",
-                                                                                      template,
-                                                                                      i,
-                                                                                      " ".join(short_subjects),
-                                                                                      template,
-                                                                                      i + 1)
+        command = f"FREESURFER mris_make_template {'lh'} sphere.{template}{i} {' '.join(short_subjects)} " \
+                  f"lh{template}{i + 1}.tif"
         print(command)
         os.system(command)
 
@@ -188,7 +182,7 @@ def make_group_average(subjects=None, subject_dir=None, average=None, hemi="lh",
     make_average_subject(subjects=subjects,
                          subject_dir=subject_dir,
                          average_dir=average_dir,
-                         fn_reg='sphere.{}{}'.format(template, steps-1))
+                         fn_reg='sphere.{}{}'.format(template, steps - 1))
 
 
 def make_average_subject(subjects, subject_dir, average_dir, fn_reg='sphere.reg'):
@@ -224,7 +218,7 @@ def make_average_subject(subjects, subject_dir, average_dir, fn_reg='sphere.reg'
     # check if FREESURFER is installed
     output = bash('which FREESURFER')
     if not output:
-        raise Exception('FREESURFER not installed! Please visit http://freesurfer.net/ for more information')
+        raise Exception('FREESURFER not installed! Please visit https://freesurfer.net/ for more information')
 
     # define $SUBJECT_DIR of FREESURFER
     if not os.path.exists(subject_dir):
@@ -240,8 +234,8 @@ def make_average_subject(subjects, subject_dir, average_dir, fn_reg='sphere.reg'
             bash('ln -s ' + subject + ' ' + subject_dir)
 
     # make average template
-    if os.path.exists(subject_dir + '/avg_template'):
-        shutil.rmtree(subject_dir + '/avg_template')
+    if os.path.exists(f'{subject_dir}{os.sep}avg_template'):
+        shutil.rmtree(f'{subject_dir}{os.sep}avg_template')
     print(('Generating average subject in {}/avg_template'.format(subject_dir)))
     print('==============================================')
     command = 'FREESURFER make_average_subject ' \
@@ -324,20 +318,20 @@ def data_sub2avg(fn_subject_obj, fn_average_obj, hemisphere, fn_in_hdf5_data, da
 
     # filename of mapped subject data in nodes in .curv format for freesurfer (will be generated)
     fn_data_sub_nodes_mapped_curv = [os.path.join(
-                                     os.path.split(fn_out_hdf5_geo)[0],
-                                     hemisphere + '.' +
-                                     os.path.split(subject_dir)[1] +
-                                     '_' +
-                                     data_label[i] +
-                                     '.curv')
-                                     for i in range(len(data_label))]
+        os.path.split(fn_out_hdf5_geo)[0],
+        hemisphere + '.' +
+        os.path.split(subject_dir)[1] +
+        '_' +
+        data_label[i] +
+        '.curv')
+        for i in range(len(data_label))]
 
     # filename of mapped subject data in nodes on average template in .curv format for freesurfer (will be generated)
     fn_data_sub_nodes_mapped_avg_curv = [os.path.join(
-                                    os.path.split(fn_data_sub_nodes_mapped_curv[i])[0],
-                                    os.path.splitext(os.path.split(fn_data_sub_nodes_mapped_curv[i])[1])[0] +
-                                    '_avg.curv')
-                                    for i in range(len(data_label))]
+        os.path.split(fn_data_sub_nodes_mapped_curv[i])[0],
+        os.path.splitext(os.path.split(fn_data_sub_nodes_mapped_curv[i])[1])[0] +
+        '_avg.curv')
+        for i in range(len(data_label))]
 
     if verbose:
         print('> Generating GM/WM surface on average template')
@@ -346,7 +340,8 @@ def data_sub2avg(fn_subject_obj, fn_average_obj, hemisphere, fn_in_hdf5_data, da
                                              mesh_folder=mesh_folder_avg,
                                              delta=subject.roi[mesh_idx][roi_idx]['delta'],
                                              layer=1)
-    files_not_exist = not np.array([os.path.exists(fn_data_sub_nodes_mapped_avg_curv[i]) for i in range(len(data_label))]).all()
+    files_not_exist = not np.array([os.path.exists(fn_data_sub_nodes_mapped_avg_curv[i]) for
+                                    i in range(len(data_label))]).all()
 
     if files_not_exist or replace:
 
@@ -424,13 +419,13 @@ def data_sub2avg(fn_subject_obj, fn_average_obj, hemisphere, fn_in_hdf5_data, da
             print('> Mapping subject data to average template using freesurfer mri_surf2surf')
         for i in range(len(data_label)):
             fs_cmd = "FREESURFER mri_surf2surf --srcsubject " + os.path.split(subject_dir)[1] + \
-                                             " --trgsubject " + os.path.split(average_dir)[1] + \
-                                             " --srcsurfval " + fn_data_sub_nodes_mapped_curv[i] + \
-                                             " --trgsurfval " + fn_data_sub_nodes_mapped_avg_curv[i] + \
-                                             " --hemi "       + hemisphere + \
-                                             " --src_type curv " + \
-                                             " --trg_type curv " + \
-                                             " --surfreg " + reg_fn
+                     " --trgsubject " + os.path.split(average_dir)[1] + \
+                     " --srcsurfval " + fn_data_sub_nodes_mapped_curv[i] + \
+                     " --trgsurfval " + fn_data_sub_nodes_mapped_avg_curv[i] + \
+                     " --hemi " + hemisphere + \
+                     " --src_type curv " + \
+                     " --trg_type curv " + \
+                     " --surfreg " + reg_fn
 
             os.system(fs_cmd)
 
