@@ -7,7 +7,9 @@ import time
 import h5py
 import nibabel as nib
 import trimesh
+import tqdm
 from pynibs.util import sample_sphere
+from itertools import product
 from simnibs import SIMNIBSDIR, mesh_io, transformations
 
 
@@ -141,7 +143,7 @@ def load_roi_surface_obj_from_hdf5(fname):
 
     with h5py.File(fname, 'r') as f:
         if "roi_surface" not in f.keys():
-            print(f"Not rois found in {fname}.")
+            print(f"No rois found in {fname}.")
             return None
         roi_label = list(f["roi_surface"].keys())
         roi_label = [str(r) for r in roi_label]
@@ -585,11 +587,11 @@ class RegionOfInterestSurface:
     tri_center_coord_low : nparray [N_tri x 3]
         Coordinates of roi triangle center of lower surface
     fn_mask : string
-        Filename for .mgh mask in subject space.
+        Filename for surface mask in subject space. .mgh file or freesurfer surface file.
     fn_mask_avg : string
-        Filename for .mgh mask in fsaverage space.
-    fn_mask : string
-        Filename for .nii or .nii.gz mask.
+        Filename for .mgh mask in fsaverage space. Absolute path or relative to mesh folder.
+    fn_mask_nii : string
+        Filename for .nii or .nii.gz mask. Absolute path or relative to mesh folder.
     X_ROI : list of float
         Region of interest [Xmin, Xmax], whole X range if empty [0,0] or None
         (left - right)
@@ -600,7 +602,7 @@ class RegionOfInterestSurface:
         Region of interest [Zmin, Zmax], whole Z range if empty [0,0] or None
         (inferior - superior)
     template : str
-        'MNI', 'fsaverage', 'subject
+        'MNI', 'fsaverage', 'subject'
     center : list of float
         Center coordinates for spherical ROI in self.template space
     radius : float
@@ -613,7 +615,6 @@ class RegionOfInterestSurface:
         (e.g. in mri2msh: .../fs_ID/surf/lh.white)
     layer : int
         Define the number of layers:
-
         - 1: one layer
         - 3: additionally upper and lower layers are generated around the central midlayer
 
@@ -644,16 +645,16 @@ class RegionOfInterestSurface:
         self.tri_center_coord_mid = []
         self.tri_center_coord_low = []
 
+        self.template = None
         self.fn_mask = []
         self.fn_mask_avg = None
-        self.fn_mask_mni = None
+        self.fn_mask_nii = None
 
         self.X_ROI = []
         self.Y_ROI = []
         self.Z_ROI = []
 
-        self.center_mni = None
-        self.center_sub = None
+        self.center = None
         self.radius = None
 
         self.gm_surf_fname = []
@@ -809,40 +810,47 @@ class RegionOfInterestSurface:
         points_wm = [None for _ in range(len(wm_surf_fname))]
         points_mid = [None for _ in range(len(midlayer_surf_fname))]
         con_gm = [None for _ in range(len(gm_surf_fname))]
-        con_wm = [None for _ in range(len(wm_surf_fname))]
+        # con_wm = [None for _ in range(len(wm_surf_fname))]
         con_mid = [None for _ in range(len(midlayer_surf_fname))]
 
         max_idx_gm = 0
-        max_idx_wm = 0
+        # max_idx_wm = 0
         max_idx_mid = 0
 
+        def read_geom(fn, max_idx):
+            """
+            Read freesurfer geometries, either in .central format or as .gii
+            """
+            if not fn.startswith(os.sep):
+                fn = os.path.join(mesh_folder, fn)
+
+            # charm uses .gii files
+            if fn.endswith('.gii'):
+                points, con = nib.load(fn).agg_data()
+
+            # headreco uses .central files
+            else:
+                points, con = nib.freesurfer.read_geometry(fn)
+
+            con += max_idx
+            max_idx += points.shape[0]  # np.max(con_gm[i]) + 2
+            return points, con, max_idx
+
         for i in range(len(gm_surf_fname)):
             if gm_surf_fname[i] is not None:
-                if not gm_surf_fname[i].startswith(os.sep):
-                    gm_surf_fname[i] = os.path.join(mesh_folder, gm_surf_fname[i])
-                points_gm[i], con_gm[i] = nib.freesurfer.read_geometry(gm_surf_fname[i])
-                con_gm[i] = con_gm[i] + max_idx_gm
-                max_idx_gm = max_idx_gm + points_gm[i].shape[0]  # np.max(con_gm[i]) + 2
+                points_gm[i], con_gm[i], max_idx_gm = read_geom(gm_surf_fname[i], max_idx_gm)
 
             if wm_surf_fname[i] is not None:
-                if not wm_surf_fname[i].startswith(os.sep):
-                    wm_surf_fname[i] = os.path.join(mesh_folder, wm_surf_fname[i])
-                points_wm[i], con_wm[i] = nib.freesurfer.read_geometry(wm_surf_fname[i])
-                con_wm[i] = con_wm[i] + max_idx_wm
-                max_idx_wm = max_idx_wm + points_wm[i].shape[0]  # np.max(con_wm[i]) + 2
+                points_wm[i], _, max_idx_wm = read_geom(wm_surf_fname[i], max_idx_gm)
 
             if midlayer_surf_fname[i] is not None:
-                if not midlayer_surf_fname[i].startswith(os.sep):
-                    midlayer_surf_fname[i] = os.path.join(mesh_folder, midlayer_surf_fname[i])
-                points_mid[i], con_mid[i] = nib.freesurfer.read_geometry(midlayer_surf_fname[i])
-                con_mid[i] = con_mid[i] + max_idx_mid
-                max_idx_mid = max_idx_mid + points_mid[i].shape[0]  # np.max(con_wm[i]) + 2
+                points_mid[i], con_mid[i], max_idx_mid = read_geom(midlayer_surf_fname[i], max_idx_mid)
 
         points_gm = np.vstack(points_gm)
         points_wm = np.vstack(points_wm)
         points_mid = np.vstack(points_mid)
         con_gm = np.vstack(con_gm)
-        con_wm = np.vstack(con_wm)
+        # con_wm = np.vstack(con_wm) # we don't use this
         con_mid = np.vstack(con_mid)
 
         # Determine 3 layer midlayer if GM and WM surfaces are present otherwise use provided midlayer data
@@ -1340,7 +1348,8 @@ class RegionOfInterestVolume:
 
 
 def get_sphere_in_nii(center, radius, nii=None, out_fn=None,
-                      thresh_by_nii=True, val_in=1, val_out=0):
+                      thresh_by_nii=True, val_in=1, val_out=0,
+                      outside_val=0, outside_radius=np.inf):
     """
     Computes a spherical ROI for a given Nifti image (defaults to simnibs MNI T1 tissue). The ROI area is defined
     in nifti coordinates.
@@ -1358,6 +1367,10 @@ def get_sphere_in_nii(center, radius, nii=None, out_fn=None,
         The nifti image to work with.
     out_fn : string, optional
         If provided, sphere ROI image is saved here
+    outside_val : float, default = None
+        Value outside of outside_radius.
+    outside_radius : float, default = None
+        Distance factor to define the 'outside' area: oudsidefactor * radius -> outside
 
     Returns
     -------
@@ -1378,7 +1391,8 @@ def get_sphere_in_nii(center, radius, nii=None, out_fn=None,
     ValueError
         If the final ROI is empty.
     """
-
+    if outside_radius is None:
+        outside_radius = np.inf
     # load image for affine and tissue borders
     if nii is None:
         mni_fn = f"{SIMNIBSDIR}/resources/templates/spmprior_tissue.nii"
@@ -1386,38 +1400,132 @@ def get_sphere_in_nii(center, radius, nii=None, out_fn=None,
     elif isinstance(nii, str):
         nii = nib.load(nii)
 
-    center = np.atleast_1d(center)
+    center = np.atleast_1d(center).astype(int)
     assert center.shape == (3,)
 
-    roi_mni_data = np.zeros_like(nii.get_fdata())
-    roi_mni_data[:] = val_out
+    roi_data = np.zeros_like(nii.get_fdata())
+    roi_data[:] = val_out
 
     center = np.atleast_1d(center)
+    # radius = np.array(radius).astype(int)
+
+    if outside_radius != np.inf:
+        # searach space is whole nifti space
+        x_min, y_min, z_min = nib.affines.apply_affine(nii.affine, [0, 0, 0]).astype(int)
+        x_max, y_max, z_max = nib.affines.apply_affine(nii.affine, list(nii.shape[:3])).astype(int)
+    else:
+        # if outside_val should not be set we only need to search within center-radius
+        x_min, y_min, z_min = center - radius
+        x_max, y_max, z_max = center + radius
+
+    # speed up the long loop below
+    affine_inv = np.linalg.inv(nii.affine)
+    apply_affine = nib.affines.apply_affine
+    norm = np.linalg.norm
 
     # compute spherical roi around nifti coordinates
     # do this in original coordinates to have correct mm radius
-    for x in range(center[0] - radius, center[0] + radius):
-        for y in range(center[1] - radius, center[1] + radius):
-            for z in range(center[2] - radius, center[2] + radius):
-                # set to val_in if within radius
-                if np.linalg.norm(center - np.array([x, y, z])) < radius:
-                    vox = nib.affines.apply_affine(np.linalg.inv(nii.affine), [x, y, z]).astype(int)
-                    # print(f"{x}, {y}, {z} -- {vox}: {np.linalg.norm(center - np.array([x,y,z]))}")
-                    roi_mni_data[vox[0], vox[1], vox[2]] = val_in
+    for xyz in tqdm.tqdm(product(range(x_min, x_max), range(y_min, y_max), range(z_min, z_max)),
+                         total=(x_max-x_min)*(y_max-y_min)*(z_max-z_min),
+                         desc="Finding ROI elements."):
+        # set to val_in if within radius
+        dist = norm(center - xyz)
+        if dist < radius:
+            vox_x, vox_y, vox_z = apply_affine(affine_inv, xyz).astype(int)
+            roi_data[vox_x, vox_y, vox_z] = val_in
+        elif dist > outside_radius:
+            # Set values outside of outside_radius to outside_val
+            vox_x, vox_y, vox_z = apply_affine(affine_inv, xyz).astype(int)
+            roi_data[vox_x, vox_y, vox_z] = outside_val
 
     # check if there's something left in the ROI
-    if np.sum(roi_mni_data) == 0:
+    if np.sum(roi_data) == 0:
         raise ValueError(f"No tissue found in {radius} mm sphere around {center}.")
 
     # threshold roi by tissue
     if thresh_by_nii:
-        roi_mni_data[nii.get_fdata() == 0] = 0
+        roi_data[nii.get_fdata() == 0] = 0
+        roi_data[np.isnan(nii.get_fdata())] = 0
+
+    roi_mni_img = nib.Nifti1Image(roi_data, nii.affine)
 
     if out_fn is not None:
         # make new roi image and save file
-        roi_mni_img = nib.Nifti1Image(roi_mni_data, nii.affine)
         nib.save(roi_mni_img, out_fn)
 
+    return roi_mni_img
+
+
+def create_refine_spherical_roi(center, radius, final_tissues_nii, out_fn, target_size=.5,
+                                outside_size=None, outside_factor=3,
+                                out_spher_fn=None, tissue_types=None, verbose=False):
+    """
+    Create a spherical roi nifti for simnibs 4 refinement.
+    Only tissue types accoring to _tissue_types will be refined.
+
+    Use the resulting output file as input for --sizing_field in SimNIBS-4/simnibs/cli/meshmesh.py
+
+    Paraneters
+    ----------
+    center : list of float
+        Center of spherical ROI in mm
+    radius : float
+        Radius of spherical ROI in mm
+    final_tissues_nii : string or nib.nifti1.Nifti1Image
+        final_tissues.nii.gz to create roi for.
+    out_fn : str
+        Final output filename
+    target_size : float, default = 0.5
+        Target element size of refined areas in mm (?)
+    outside_size : float, default = None
+        Element size outside of target size.
+    outside_factor : float, default = None
+        Distance factor to define the 'outside' area: oudsidefactor * radius -> outside
+
+    Other parameters
+    ----------------
+    out_spher_fn : str, optional
+        Output filename of orignal, raw spherical ROI
+    tissue_types : list of float, default = [1,2,3]
+        Which tissue types to refine. Defaults to WM, GM, CSF
+    verbose : bool, optional, default=False
+        Print additional information
+    """
+    # get spherical ROI, masked to all tissues
+    roi_img = get_sphere_in_nii(
+        center=center,
+        radius=radius,
+        nii=final_tissues_nii,
+        val_in=target_size,
+        out_fn=out_spher_fn,
+        outside_val=outside_size, outside_radius=radius * outside_factor)
+
+    # get tissue_types data
+    if isinstance(final_tissues_nii, str):
+        final_tissues_nii = nib.load(final_tissues_nii)
+    org_img_data = final_tissues_nii.get_fdata()
+
+    if tissue_types is None:
+        tissue_types = [1, 2, 3]
+
+    data = roi_img.get_fdata()
+    if verbose:
+        print(f"{np.sum(data == target_size): >6} elements found in spherical roi.")
+        if outside_size is not None:
+            print(f"{np.sum(data == outside_size): >6} elements outside area in spherical roi.")
+    # apply tissue_types list mask
+    data[~np.isin(org_img_data, tissue_types) & (data == target_size)] = 0
+
+    # apply refined element size
+    # data[data == 1] = target_size
+
+    if verbose:
+        print(f"{np.sum(data != 0): >6} elements found in spherical roi for tissues {tissue_types}.")
+
+    # write final image
+    roi_refine_img = nib.Nifti2Image(affine=roi_img.affine, dataobj=data)
+    roi_refine_img.to_filename(out_fn)
+
 
 def clean_roi(img, vox_thres=.5, fn_out=None):
     """
@@ -1499,4 +1607,4 @@ def nii2msh(mesh, m2m_dir, nii, out_folder, hem, out_fsaverage=False, roi_name='
     transformations.middle_gm_interpolation(fn_mesh_out, m2m_folder=m2m_dir,
                                             out_folder=out_folder, out_fsaverage=out_folder)
 
-    return nib.freesurfer.read_morph_data(os.path.join(out_folder, f"{hem}.mesh.central.{roi_name}"))
\ No newline at end of file
+    return nib.freesurfer.read_morph_data(os.path.join(out_folder, f"{hem}.mesh.central.{roi_name}"))
