@@ -1,10 +1,5 @@
 import numpy as np
-from pynibs.pckg import gdist
-#try:
-#    from pynibs.pckg import gdist
-#except ImportError:
-#    print("Could not import gdist from pynibs.pckg. Geodesic distance measures will not work.")
-import time
+import gdist
 
 
 def geodesic_dist(nodes, tris, source, source_is_node=True):
